/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sql.service;

import java.io.IOException;
import java.io.OutputStream;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.ow2.petals.bc.sql.Constants;
import org.ow2.petals.commons.threadlocal.XMLOutputFactories;

/**
 * 
 * @author alouis
 * 
 */
public class SQLService {

    private Logger logger;

    public SQLService(Logger logger) {
        super();
        this.logger = logger;
    }

    /**
     * Execute the select SQL command on the given connection.
     * 
     * @param sql
     * @param connection
     * @return
     * @throws SQLException
     */
    public ResultSet select(String sql, Connection connection) throws SQLException {
        Statement statement = connection.createStatement();
        logger.info("Execute the following SQL statement : " + sql);

        return statement.executeQuery(sql);
    }

    /**
     * Execute the SQL Stored Procedure on the given connection.
     * 
     * @param sql
     * @param connection
     * @return
     * @throws SQLException
     * @throws ParseException
     * @throws MessagingException
     * @throws IOException
     */
    public void storedProcedure(List<SQLParameter> sqlParameters, String call, Connection connection)
            throws SQLException, ParseException, MessagingException, IOException {
        CallableStatement statement = connection.prepareCall(call);

        for (SQLParameter sqlParameter : sqlParameters) {
            if (SQLParameter.IN_SCOPE.equals(sqlParameter.getScope())
                    || SQLParameter.INOUT_SCOPE.equals(sqlParameter.getScope())) {
                logger.finest("Register a SQL stored procedure parameter : " + sqlParameter);
                SQLParameter.setSQLParameter(statement, sqlParameter, sqlParameter.getRank());
            }

            if (!SQLParameter.IN_SCOPE.equals(sqlParameter.getScope())) {
                logger
                        .finest("Register SQL stored procedure parameter to return : "
                                + sqlParameter);
                statement.registerOutParameter(sqlParameter.getRank(), sqlParameter.getType());
            }
        }

        logger.info("Execute the following SQL stored procedure : " + call);
        statement.execute();

        for (SQLParameter sqlParameter : sqlParameters) {
            if (SQLParameter.OUT_SCOPE.equals(sqlParameter.getScope())
                    || SQLParameter.INOUT_SCOPE.equals(sqlParameter.getScope())) {
                SQLParameter.getSQLParameter(statement, sqlParameter, sqlParameter.getRank());
                logger.finest("Set a returned SQL stored procedure parameter : " + sqlParameter);

            }
        }

    }

    /**
     * Execute the update SQL command on the given connection.
     * 
     * @param sql
     * @param connection
     * @return
     * @throws SQLException
     */
    public int update(String sql, Connection connection) throws SQLException {
        Statement statement = connection.createStatement();
        logger.info("Execute the following SQL instruction : " + sql);

        return statement.executeUpdate(sql);
    }

    /**
     * Transform the given resultSet as an XML outputStream
     * 
     * @param resultSet
     *            the jdbc resultSet to transform
     * @param outputStream
     *            the stream where the XML is written
     * @param showAttributes
     *            row index, column name and type are set as xml attributes in
     *            the response
     * @throws SQLException
     * @throws XMLStreamException
     */
    public void resultSetAsXMLOutputStream(ResultSet resultSet, OutputStream outputStream,
            boolean showAttributes) throws SQLException, XMLStreamException {
        XMLStreamWriter writer = XMLOutputFactories.getDefaultXMLOutputFactory()
                .createXMLStreamWriter(outputStream);
        writer.writeStartDocument();
        writer.writeStartElement(Constants.RESULT_ELEMENT);
        writer.writeDefaultNamespace(Constants.NAMESPACE);

        if (resultSet != null) {
            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();
            // resultSet.beforeFirst();
            while (resultSet.next()) {
                writer.writeStartElement(Constants.ROW_ELEMENT);
                if (showAttributes) {
                    writer.writeAttribute(Constants.INDEX_ATTRIBUTE, Integer.toString(resultSet
                            .getRow()));
                }
                for (int i = 1; i <= columnCount; i++) {
                    writer.writeStartElement(Constants.COLUMN_ELEMENT);
                    if (showAttributes) {
                        writer.writeAttribute(Constants.NAME_ATTRIBUTE, metaData.getColumnName(i));
                        writer.writeAttribute(Constants.TYPE_ATTRIBUTE, metaData
                                .getColumnTypeName(i));
                    }
                    final String resultvalue = resultSet.getString(i);
                    if (resultvalue == null) {
                        writer.writeCharacters("NULL");
                    } else {
                        writer.writeCharacters(resultvalue);
                    }
                    writer.writeEndElement();
                }
                writer.writeEndElement();
            }
        }
        writer.writeEndElement();
        writer.writeEndDocument();
    }

    /**
     * Inject the given list of SQL parameters into a XML outputStream.
     * 
     * @param sqlParameters
     *            the list of SQL parameters from the Stored Statement
     * @param outputStream
     *            the stream where the XML is written
     * @param isResultXML
     *            if {@code true},
     * 
     * @throws XMLStreamException
     * @throws IOException
     */
    public void sqlParametersAsXMLOutputStream(List<SQLParameter> sqlParameters,
            OutputStream outputStream, boolean isResultXML) throws XMLStreamException, IOException {

        final XMLStreamWriter writer = XMLOutputFactories.getDefaultXMLOutputFactory()
                .createXMLStreamWriter(outputStream);
        writer.writeStartDocument();
        writer.writeStartElement(Constants.STOREDPROCEDURE_RESULT_ELEMENT);
        writer.writeDefaultNamespace(Constants.NAMESPACE);

        // process XMLStoredProcedure response message
        if (isResultXML) {
            for (SQLParameter sqlParameter : sqlParameters) {
                if (SQLParameter.INOUT_SCOPE.equals(sqlParameter.getScope())
                        || SQLParameter.OUT_SCOPE.equals(sqlParameter.getScope())) {
                    writer.writeStartElement(Constants.PARAMETER_ELEMENT);
                    writer.writeAttribute(Constants.RANK_ATTRIBUTE, Short.toString(sqlParameter
                            .getRank()));
                    if (sqlParameter.getValue() == null) {
                        writer.writeCharacters("NULL");
                    } else {
                        writer.writeCharacters(sqlParameter.getValue());
                    }
                    writer.writeEndElement();
                }
            }

        } else {
            // process StoredProcedure response message
            final Iterator<SQLParameter> sqlParameterIterator = sqlParameters.iterator();
            while (sqlParameterIterator.hasNext()) {
                final SQLParameter sqlParameter = sqlParameterIterator.next();
                if (SQLParameter.INOUT_SCOPE.equals(sqlParameter.getScope())
                        || SQLParameter.OUT_SCOPE.equals(sqlParameter.getScope())) {
                    writer.writeCharacters(sqlParameter.getRank() + ":" + sqlParameter.getScope()
                            + ":" + sqlParameter.getValue());
                }
                if (sqlParameterIterator.hasNext()) {
                    writer.writeCharacters(";");
                }
            }
        }

        writer.writeEndElement();
        writer.writeEndDocument();

    }

}
